@@ -27,7 +27,7 @@ SIGN_IN_URL="{}/signIn".format(url)
 CREATE_MODEL_URL="{}/createModelStream".format(url)
 UPLOAD_URL="{}/userUploadStream".format(url)
 STATUS_URL="{}/status".format(url)
-TRAIN_URL="{}/train".format(url)
+TRAIN_URL="{}/trainStream".format(url)
 INFER_URL="{}/inferStream".format(url)
 INFER_ASYNC_URL="{}/inferAsyncStream".format(url)
 RESULTS_URL="{}/results".format(url)
